#include <console.h>

Console::Console(QWidget *parent) : QWidget(parent)
{
	this->ui = new Ui::Console();
	this->ui->setupUi(this);

	const QRect screen = QApplication::desktop()->screenGeometry();
	this->move( screen.center() - this->rect().center() );

	this->serial = new QSerialPort();

	QSerialPortInfo portInfo;
	foreach(portInfo, QSerialPortInfo::availablePorts())
		this->ui->box_port->addItem(portInfo.systemLocation());

	QObject::connect(this->ui->btn_connect, SIGNAL(toggled(bool)), this, SLOT(btnConnectToggled(bool)));
	QObject::connect(this->serial, SIGNAL(readyRead()), this, SLOT(receive()));
}

Console::~Console()
{
	delete this->ui;
	delete this->serial;
}

void Console::btnConnectToggled(bool checked)
{
	if(checked)
		this->connect();
	else
		this->disconnect();
}

void Console::connect()
{
	bool success(false);

	QString portName = this->ui->box_port->currentText();
	this->serial->setPortName(portName);
	success = this->serial->open(QIODevice::ReadOnly);

	if(success)
	{
		this->ui->btn_connect->setText(QString("Disconnect"));
		this->ui->txt_output->appendPlainText(QString("Connection established!"));

		success = this->serial->setBaudRate(QSerialPort::Baud9600);

		if(!success)
		{
			this->ui->txt_output->appendPlainText(QString("Fail to set baud rate."));
			this->ui->btn_connect->setChecked(false);
			return;
		}

		success = this->serial->setDataBits(QSerialPort::Data8);

		if(!success)
		{
			this->ui->txt_output->appendPlainText(QString("Fail to set data bits."));
			this->ui->btn_connect->setChecked(false);
			return;
		}

		success = this->serial->setStopBits(QSerialPort::OneStop);

		if(!success)
		{
			this->ui->txt_output->appendPlainText(QString("Fail to set stop bits."));
			this->ui->btn_connect->setChecked(false);
			return;
		}

		success = this->serial->setFlowControl(QSerialPort::NoFlowControl);

		if(!success)
		{
			this->ui->txt_output->appendPlainText(QString("Fail to set flow control."));
			this->ui->btn_connect->setChecked(false);
			return;
		}

		this->ui->btn_connect->setText(QString("Disconnect"));
	}
	else
	{
		this->ui->txt_output->appendPlainText(QString("Fail to open %0.").arg(portName));
		this->ui->btn_connect->setChecked(false);
	}
}

void Console::disconnect()
{
	this->ui->btn_connect->setText(QString("Connect"));
	this->ui->txt_output->appendPlainText(QString("Connection finished."));
	this->serial->close();
}

void Console::receive()
{
	QString data;
	while(this->serial->bytesAvailable())
	{
		data.append(QString(this->serial->read(3)).toUtf8());
	}

	this->ui->txt_output->insertPlainText(data);
}
