#include <cbuffer.h>

cbuffer_t* cbuffer_create(int size)
{
	cbuffer_t *cb = kmalloc(sizeof(cbuffer_t));

	cb->data = kmalloc(size+1);
	cb->size = size+1;
	cb->start = 0;
	cb->end = 0;

	return cb;
}

void cbuffer_destroy(cbuffer_t *cb)
{
	kfree(cb->data);
	kfree(cb);
}

int cbuffer_full(cbuffer_t *cb)
{
	return ((cb->end + 1) % cb->size) == cb->start;
}

int cbuffer_empty(cbuffer_t *cb)
{
	return cb->end == cb->start;
}

int cbuffer_size(cbuffer_t *cb)
{
	return (cb->size-1);
}

int cbuffer_available(cbuffer_t *cb)
{
	int available;

	if(cb->start <= cb->end)
		available = cb->end - cb->start;
	else
		available = cb->size - (cb->start - cb->end);

	return available;
}

char cbuffer_read(cbuffer_t *cb)
{
	char ch;
	ch = cb->data[cb->start];
	cb->start = (cb->start + 1) % cb->size;
	return ch;
}

void cbuffer_write(cbuffer_t *cb, char ch)
{
	cb->data[cb->end] = ch;
	cb->end = (cb->end + 1) % cb->size;
	if(cbuffer_empty(cb))
		cb->start = (cb->start + 1) % cb->size;
}

