#ifndef __DELTA_QUEUE_H__
#define __DELTA_QUEUE_H__

#include <libc.h>
#include <stdint.h>
#include <string.h>

/*
	Delta Queue
*/

#define new(x) kmalloc(x)
#define delete(x) kfree(x)

typedef struct DeltaQueueItem
{
	void *data;
	int delta;
	void *prev;
	void *next;

} DeltaQueueItem;

typedef struct DeltaQueue
{
	int size;
	DeltaQueueItem *first;
	DeltaQueueItem *last;

} DeltaQueue;

DeltaQueue* deltaQueueCreate();
int deltaQueueSize(DeltaQueue *dQueue);
void deltaQueueEnqueue(DeltaQueue *dQueue, void *data, int delta);
void* deltaQueueDequeue(DeltaQueue *dQueue);
DeltaQueueItem* deltaQueueItemCreate(void *data, int delta);
void deltaQueuePush(DeltaQueue *dQueue, DeltaQueueItem *item);
void deltaQueueInsertBefore(DeltaQueue *dQueue, DeltaQueueItem *item, DeltaQueueItem *cursor);
void deltaQueuePushBack(DeltaQueue *dQueue, DeltaQueueItem *item);
DeltaQueueItem* deltaQueueFront(DeltaQueue *dQueue);
DeltaQueueItem* deltaQueuePopFront(DeltaQueue *dQueue);

#endif /* __DELTA_QUEUE_H__ */
