#include <exec.h>

extern void* __speaker();

void process_section_destroy(segment_t *section)
{
	uint ttb = getTranslationTableBase0();
	uint virt_addr = section->virt_addr;
	uint frame;
	uint frames = (section->offset >> 12) + (((section->offset & 0xfff) > 0)? 1 : 0);

	while(frames--)
	{
		frame = mmu_unmap_smallpage(ttb, virt_addr);
		mmap_free(frame);
		virt_addr += SMALL_PAGE_SIZE;
	}

	kfree(section);
}

void process_section_create(segment_t *section)
{
	uint ttb = getTranslationTableBase0();
	uint frame;
	uint virt_addr = section->virt_addr;
	uint phys_addr = section->phys_addr;
	uint frames = (section->offset >> 12) + (((section->offset & 0xfff) > 0)? 1 : 0);

	while(frames--)
	{
		frame = mmap_alloc_user();

		physical_frame_copy(frame, phys_addr);

		mmu_map_smallpage(ttb, frame, virt_addr, section->attr);

		virt_addr += SMALL_PAGE_SIZE;
		phys_addr += SMALL_PAGE_SIZE;
	}
}

extern void loadContext(pcb_t* pcb);

/*
	exec():
	=======
	
	Reemplaza el espacio virtual de un proceso por el de otro.
	Recibe como parámetro un número que indica a que proceso debe cambiar.
*/

int exec(int command)
{
	process_t *this = scheduler_running();
	process_t *exec = kmalloc(sizeof(process_t));

	exec->text = 0;
	exec->rodata = 0;
	exec->data = 0; 
	exec->bss = 0;
        
	switch(command)
	{
	case EXEC_SPEAKER:
		Elf32_load(exec, __speaker);
		break;

	default:
		return -1;
	}

	// Liberar secciones del proceso antiguo
	if(this->text != 0)
		process_section_destroy(this->text);

	if(this->data != 0)
		process_section_destroy(this->data);

	if(this->rodata != 0)
		process_section_destroy(this->rodata);

	if(this->bss != 0)
		process_section_destroy(this->bss);

	// Reemplazar las secciones del proceso actual por las del nuevo.
	this->text = exec->text;
	this->rodata = exec->rodata;
	this->data = exec->data;
	this->bss = exec->bss;

	kfree(exec);

	// Crear y cargar en el directorio las nuevas secciones.

	if(this->text != 0)
		process_section_create(this->text);

	if(this->data != 0)
		process_section_create(this->data);

	if(this->rodata != 0)
		process_section_create(this->rodata);

	if(this->bss != 0)
		process_section_create(this->bss);

	// Liberar memoria usada en el proceso anterior
	heapFreeAll((Heap *) PROCESS_HEAP);

	// Restituir pc, stacks y status
	this->pcb->r13_usr = this->usr_stack->virt_addr + SMALL_PAGE_SIZE;
	this->pcb->r13_svc = this->svc_stack->virt_addr + SMALL_PAGE_SIZE;
	this->pcb->r14_svc = this->text->virt_addr;
	this->pcb->spsr_svc = 0x150; // PSR: Usr32md | IRQ | no FIQ | ARM state

	// debug("%% process(%d) %%\n", this->pid);
	// debug_ttb(this->pcb->ttb);
	// debug_pcb(this->pcb);

	// Saltar al proceso
	loadContext(this->pcb);

	// nunca regresa
	return 0;
}
