#ifndef __HEAP_H__
#define __HEAP_H__

#include <debug.h>
#include <mmu.h>
#include <stdint.h>

/*
	Implementación de Heap. Utilizada por kernel y tareas.
*/

#define MaxIterations 2

typedef struct HeapNode
{
	uint32_t size;
	void *prev;
	void *next;
	
} HeapNode;

typedef struct Heap
{
	HeapNode *freeByAddress;
	HeapNode *used;
	
	uint32_t start, end;
	
} Heap;

Heap* heapInit(uint32_t ttb, uint32_t vAddress);
void heapDeleteFree(Heap *heap, HeapNode *node);
void heapInsertFree(Heap *heap, HeapNode *node);
void heapDeleteUsed(Heap *heap, HeapNode *node);
void heapInsertUsed(Heap *heap, HeapNode *node);
HeapNode* heapSplitNode(Heap *heap, HeapNode *node, uint32_t size);
void* heapAllocateMemory(Heap *heap, uint32_t size);
void heapMergeNode(Heap *heap, HeapNode *node);
void heapFreeMemory(Heap *heap, void *address);
void heapFreeAll(Heap *heap);

/*
 * Debug
 */

void debugHeap(Heap *heap);

#endif /* __HEAP_H__ */
