#ifndef __IVT_H__
#define __IVT_H__

#include <arm.h>
#include <debug.h>
#include <uart.h> 
#include <scheduler.h>
#include <sleep.h>
#include <stdint.h>
#include <syscall.h>
#include <timer.h>

#define InterruptVectorTableAddress 0x0

#define Reset 0
#define UndefinedInstruction 1
#define SWI 2
#define PreFetchAbort 3
#define DataAbort 4
#define IRQ 6
#define FIQ 7

void setInterruptHandler(void (*handler)(void), int vector);

void C_undefinedInstruction(uint32_t *sp);
void C_preFetchAbort(uint32_t *sp);
void C_dataAbort(uint32_t *sp);
void C_irq();
void C_fiq();
void halt();

/*
 * Funciones declaradas en src/exceptions.R
 */

extern void __hdl_unf();
extern void __hdl_swi();
extern void __hdl_pfa();
extern void __hdl_dta();
extern void __hdl_irq();
extern void __hdl_fiq();

/*
 * Debug
 */

void debugStack(uint32_t *sp, int count);
void debugDataFaultStatus(uint32_t value);
void debugInstructionFaultStatus(uint32_t value);
void debugInterruptVectorTable(uint32_t* ivt);
void debugControlRegister(uint32_t ctrl);

#endif /* __IVT_H__ */
