#ifndef __LIST_H__
#define __LIST_H__

#include <libc.h>
#include <type.h>

#define new(x) kmalloc(x)
#define delete(x) kfree(x)

typedef struct ListItem ListItem;
typedef struct List List;

struct ListItem
{
	void *data;
	ListItem *prev;
	ListItem *next;
};

struct List
{
	int size;
	ListItem *first;
	ListItem *last;
};

List* listCreate();
void listDestroy(List *list);
int listSize(List *list);
bool listEmpty(List *list);
void* listFront(List *list);
void* listBack(List *list);
void listPushBack(List *list, void *data);
void listPushFront(List *list, void *data);
void* listPopBack(List *list);
void* listPopFront(List *list);

//~ void listUnlinkItem(List *list, ListItem *item);
//~ int listRemoveItem(List *list, void *data);

ListItem* listItemCreate(void *data);
void* listItemDestroy(ListItem *item);
void listItemPushBack(List *list, ListItem *item);
void listItemPushFront(List *list, ListItem *item);
ListItem* listItemPopBack(List *list);
ListItem* listItemPopFront(List *list);

typedef enum ListIteratorDirection ListIteratorDirection;
typedef struct ListIterator ListIterator;

enum ListIteratorDirection { LeftToRight, RightToLeft };

struct ListIterator
{
	List *list;
	ListItem *cursor;
	ListIteratorDirection direction;
};

ListIterator* listIteratorCreate(List *list);
bool listIteratorHasNext(ListIterator *iter);
bool listIteratorHasPrev(ListIterator *iter);
ListItem* listIteratorNext(ListIterator *iter);
ListItem* listIteratorPrev(ListIterator *iter);
void listIteratorReverse(ListIterator *iter);
ListIteratorDirection listIteratorGetDirection(ListIterator *iter);
void listIteratorDestroy(ListIterator *iter);

#endif /* __LIST_H__ */
