#ifndef __PROCESS_H__
#define __PROCESS_H__

#include <address.h>
#include <debug.h>
#include <elf.h>
#include <libc.h>
#include <queue.h>
#include <stdint.h>
#include <string.h>
#include <type.h>
#include <vector.h>
#include <mmu.h>

#define PSTATE_RDY 0 // Listo para ejecutar (en cola)
#define PSTATE_BLK 1 // Bloquado por una operación de I/O (sleep, mutex, etc)
#define PSTATE_RUN 2 // Actualmente en ejecución
#define PSTATE_FSH 3 // Terminado correctamente
#define PSTATE_ERR 4 // Terminado con error (desalojado por sistema)

#define PROC_QUANTUM 50

typedef struct process_t process_t;
typedef struct segment_t segment_t;
typedef struct pcb_t pcb_t;

struct process_t
{
	int pid;	 			// Identificador de proceso
	
	process_t *parent; 		// Puntero a proceso padre
	Queue *children; 		// Procesos hijos
	
	char **argv; 			// Linea de commandos (Test) ?? No utilizado
	int argc;
	
	pcb_t *pcb;				// Bloque de control de proceso
	
	uint32_t quantum; 		// Quantum (valor asignado)
	uint32_t ticks;		 	// Ticks faltantes (contador usado por scheduler)
	uint32_t state; 		// Estado de proceso

	segment_t *text; 		// Segmentos de programa
	segment_t *rodata;
	segment_t *data;
	segment_t *bss;
	segment_t *svc_stack;
	segment_t *usr_stack;
	segment_t *heap;

	List *mutexes;			// Mutex pedidos por el proceso
	segment_t *shared; 		// Memoria compartida, actualmente sólo se puede compartir 1 pág de 4k
};

struct pcb_t
{
	uint32_t r0_usr; 	// "_usr" - User mode registers
	uint32_t r1_usr;
	uint32_t r2_usr;
	uint32_t r3_usr;
	uint32_t r4_usr;
	uint32_t r5_usr;
	uint32_t r6_usr;
	uint32_t r7_usr;
	uint32_t r8_usr;
	uint32_t r9_usr;
	uint32_t r10_usr;
	uint32_t r11_usr;
	uint32_t r12_usr;
	uint32_t r13_usr;
	uint32_t r14_usr;
	uint32_t r13_svc; 	// "_svc" - Supervisor mode registers
	uint32_t r14_svc;
	uint32_t spsr_svc; 	// Process Status Register
	uint32_t ttb; 		// Translation Table Base Address
};

struct segment_t
{
	uint32_t virt_addr; // Dirección virtual
	uint32_t phys_addr; // Dirección física (dentro de archivo elf32)
	uint32_t offset; 	// offset (tamaño)
	uint32_t attr; 		// attributos (de página)
};

extern int last_pid;

process_t* process_create();
void process_destroy();

process_t* process_init();

/* Cargador de Elf32 */

int Elf32_load(process_t *process, void *file);

/* Funciones extra */

void physical_frame_copy(uint dst, uint src);

/* Procesos (incluidos en tasks.R) */

extern void __init();

/* Debug */

void debug_pcb(pcb_t *pcb);


#endif /* __PROCESS_H__ */
