#include <uart.h>

Queue *rx_process;
Queue *tx_process;

cbuffer_t *rx_buffer;
cbuffer_t *tx_buffer;

bool tx_flag;
bool rx_flag;

void uart_init()
{
	aux_peri_reg *aux = (aux_peri_reg*) aux_peri_reg_address;
	IrqController *irq = (IrqController *) IRQ_CONTROLLER_ADDR;

	irq->pending_1.irq.aux = 1; // enable aux IRQs

	rx_process = queueCreate();
	tx_process = queueCreate();

	rx_buffer = cbuffer_create(4096);
	tx_buffer = cbuffer_create(4096);

	rx_flag = false;
	// tx_flag = true;

	aux->mini_uart.interrupt_enable = 0b0111; // enable ints
	aux->mini_uart.interrupt_identify = 0xc6; // clear fifo tx/rx
}

#define UART_IRQ_RECV 4
#define UART_IRQ_SEND 2

void uart_rx_irq()
{
	aux_peri_reg *aux = (aux_peri_reg*) aux_peri_reg_address;
	
	rx_flag = false;

	if(!(aux->mini_uart.interrupt_identify & 1))
	{
		while(aux->mini_uart.interrupt_identify & UART_IRQ_RECV)
		{
			char ch =  (char) (aux->mini_uart.data & 0xff);
			
			cbuffer_write(rx_buffer, ch);

			if(rx_process != 0)
				rx_flag = true;
		}
	}

	if(rx_flag)
	{
		if(queueSize(rx_process))
		{
			scheduler_unblock(dequeue(rx_process));
			schedule();
		}
	}

}

void uart_tx_irq()
{
	aux_peri_reg *aux = (aux_peri_reg*) aux_peri_reg_address;
	
	if(!(aux->mini_uart.interrupt_identify & 1))
	{
		if(aux->mini_uart.interrupt_identify & UART_IRQ_SEND)
		{
			while(cbuffer_available(tx_buffer) > 0)
			{
				if(aux->mini_uart.line_status & 0x20)
					aux->mini_uart.data = cbuffer_read(tx_buffer) & 0xff;
				else
					break;
			}

			if(cbuffer_size(tx_buffer) - cbuffer_available(tx_buffer))
			{
				if(queueSize(tx_process))
				{
					process_t *proc = dequeue(tx_process);
					scheduler_unblock(proc);
					schedule();
				}	
			}
		}
	}
}

void uart_irq()
{
	uart_rx_irq();
	uart_tx_irq();
}

/* syscall */

char getch()
{
	if(cbuffer_available(rx_buffer) < 1)
	{
		process_t *this = scheduler_running();
		enqueue(rx_process, this);
		scheduler_block(this);
		schedule();
	}

	return cbuffer_read(rx_buffer);
}

void putch(char ch)
{
	if(cbuffer_full(tx_buffer))
	{
		process_t *this = scheduler_running();
		enqueue(tx_process, this);
		scheduler_block(this);
		schedule();
	}

	cbuffer_write(tx_buffer, ch);
}
