#include <utils.h>

/* GPIO */

void gpio_fsel(int pin, int fsel)
{
	__asm__ ("mov r0, %[pin]" :: [pin]"r"(pin));
	__asm__ ("mov r1, %[fsel]" :: [fsel]"r"(fsel));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_GPIO_FSEL));
}

void gpio_clear(int pin)
{
	__asm__ ("mov r0, %[pin]" :: [pin]"r"(pin));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_GPIO_CLEAR));
}

void gpio_set(int pin)
{
	__asm__ ("mov r0, %[pin]" :: [pin]"r"(pin));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_GPIO_SET));
}

/* Time */

void sleep(time_t msec)
{
	__asm__ ("mov r0, %[millis]" :: [millis]"r"(msec));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_SLEEP));
}

uint32_t getTimerCounter()
{
	uint32_t value = 0;
	__asm__("swi #30");
	__asm__("mov %[value], r0" : [value]"=r"(value));
	return value;
}

void usleep(time_t usec)
{
	uint32_t end = getTimerCounter() + usec;
	while(getTimerCounter() < end);
}

/* Task */

int getpid()
{
	int pid;
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_PID));
	__asm__ ("mov %[pid], r0" : [pid]"=r"(pid));
	return pid;
}

/* Fork */

int fork()
{
	int pid = -1;
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_FORK));
	__asm__ ("mov %[pid], r0" : [pid]"=r"(pid));
	return pid;
}

int exec(int cmd)
{
	int pid = -1;
        __asm__ ("mov r0, %[cmd]" :: [cmd]"r"(cmd));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_EXEC));
	__asm__ ("mov %[pid], r0" : [pid]"=r"(pid));
	return pid;
}

/* Malloc/Free */

void* malloc(int size)
{
	void* addr = 0;
        __asm__ ("mov r0, %[size]" :: [size]"r"(size));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_MALLOC));
	__asm__ ("mov %[addr], r0" : [addr]"=r"(addr));
	return addr;
}

void free(void *addr)
{
        __asm__ ("mov r0, %[addr]" :: [addr]"r"(addr));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_FREE));
}

/* Mutex */

int mutex_create()
{
	int mutex;
	__asm__("swi %[swino]":: [swino]"I"(SWI_MUTEX_CREATE));
	__asm__("mov %[mutex], r0" : [mutex]"=r"(mutex));
	return mutex;
}

void mutex_adquire(int mutex)
{
	__asm__ ("mov r0, %[mutex]" :: [mutex]"r"(mutex));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_MUTEX_ADQUIRE));
}

void mutex_release(int mutex)
{
	__asm__ ("mov r0, %[mutex]" :: [mutex]"r"(mutex));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_MUTEX_RELEASE));
}

///

char getch()
{
	char ch;
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_GETCH));
	__asm__ ("mov %[ch], r0" : [ch]"=r"(ch));
	return ch;
}

void putch(char ch)
{
	__asm__ ("mov r0, %[ch]" :: [ch]"r"(ch));
	__asm__ ("swi %[swino]":: [swino]"I"(SWI_PUTCH));
}

void* share_mem()
{
	void *mem;
	__asm__("swi %[swino]":: [swino]"I"(SWI_SHARED_CREATE));
	__asm__("mov %[mem], r0" : [mem]"=r"(mem));
	return mem;
}

const char* toHex(char *buffer, int size, uint32_t n)
{
	char temp[size];
	int temp_ix = 0, buffer_ix = 0;

	do
	{
		uint8_t c = (uint8_t) (n % 16);
		n = n / 16;

		if(c < 10)
			temp[temp_ix++] = '0' + c;
		else
			temp[temp_ix++] = 'A' + (c - 10);

	} while(n);

	buffer[buffer_ix++] = '0';
	buffer[buffer_ix++] = 'x';

	for(int i = 0; i < (8-temp_ix); i++)
		buffer[buffer_ix++] = '0';

	while(temp_ix)
		buffer[buffer_ix++] = temp[--temp_ix];

	buffer[buffer_ix] = 0;

	return buffer;
}

const char* toBin(char *buffer, int size, uint32_t n)
{
	char temp[size];
	int temp_ix = 0, buffer_ix = 0;

	do
	{
		temp[temp_ix++] = '0' + (uint8_t) (n % 2);
		n = n / 2;

	} while(n);

	buffer[buffer_ix++] = '0';
	buffer[buffer_ix++] = 'b';

	while(temp_ix)
		buffer[buffer_ix++] = temp[--temp_ix];

	buffer[buffer_ix] = 0;

	return buffer;
}

const char* toDec(char *buffer, int size, int n)
{
	char temp[size];
	int temp_ix = 0, buffer_ix = 0;

	if(n < 0)
	{
		buffer[buffer_ix++] = '-';
		n = -n;
	}

	do
	{
		temp[temp_ix++] = '0' + (uint8_t) (n % 10);
		n = n / 10;

	} while(n);

	while(temp_ix)
		buffer[buffer_ix++] = temp[--temp_ix];

	buffer[buffer_ix] = 0;

	return buffer;
}

void __print(const char *frmt, ...)
{
	va_list args;
	va_start(args, frmt);
	char temp[64];

	while(*frmt)
	{
		if(*frmt == '%')
		{
			switch(*(frmt+1))
			{
				case 'd':
					__print("%s", toDec(temp, 64, va_arg(args, int)));
					break;

				case 'x':
					__print("%s", toHex(temp, 64, (unsigned) va_arg(args, int)));
					break;

				case 'b':
					__print("%s", toBin(temp, 64, (unsigned) va_arg(args, int)));
					break;

				case 's':
					__print(va_arg(args, const char*));
					break;

				case 'c':
					putch((char) va_arg(args, int));
					break;

				default:
					putch(*frmt);
					putch(*(frmt+1));
					break;
			}

			frmt += 2;
		}
		else
		{
			putch(*frmt++);
		}
	}
	va_end(args);
}

