#ifndef __UTILS_H__
#define __UTILS_H__

#include <stdarg.h>
#include <stdint.h>
#include <syscall.h>

/* GPIO */

#define FSEL_INPUT 	0b000
#define FSEL_OUTPUT 0b001
#define FSEL_ALT0 	0b100
#define FSEL_ALT1 	0b101
#define FSEL_ALT2 	0b110
#define FSEL_ALT3 	0b111
#define FSEL_ALT4 	0b011
#define FSEL_ALT5 	0b010

void gpio_fsel(int pin, int fsel);
void gpio_clear(int pin);
void gpio_set(int pin);

/* Time */

typedef uint32_t time_t;

void sleep(time_t time);
uint32_t getTimerCounter();
void usleep(time_t time);

/* Task */

int getpid();

/* Fork */

int fork();
int exec(int cmd);

/* Malloc/Free */

void* malloc(int size);
void free(void *addr);

/* Mutex */

int mutex_create();
void mutex_adquire(int key);
void mutex_release(int key);

char getch();
void putch(char ch);

void* share_mem();

#define print(fmt, ...) \
	mutex_adquire(0); \
	__print(fmt, __VA_ARGS__); \
	mutex_release(0)

void __print(const char *frmt, ...);


#endif /* __UTILS_H__ */